@@ -36,6 +36,8 @@
 #include <string.h>
 
 #include "config.h"
+#include "pcrepp/pcre2pp.hh"
+#include "ww898/cp_utf8.hpp"
 #include "xxHash/xxhash.h"
 
 const static int TABLE_SIZE = 4095;
@@ -218,12 +220,14 @@ string_fragment::split_lines() const
     int start = this->sf_begin;
 
     for (auto index = start; index < this->sf_end; index++) {
-        if ((*this)[index] == '\n') {
+        if (this->sf_string[index] == '\n') {
             retval.emplace_back(this->sf_string, start, index + 1);
             start = index + 1;
         }
     }
-    retval.emplace_back(this->sf_string, start, this->sf_end);
+    if (retval.empty() || start < this->sf_end) {
+        retval.emplace_back(this->sf_string, start, this->sf_end);
+    }
 
     return retval;
 }
@@ -244,3 +248,141 @@ string_fragment::utf8_length() const
 
     return Ok(retval);
 }
+
+string_fragment::case_style
+string_fragment::detect_text_case_style() const
+{
+    static const auto LOWER_RE
+        = lnav::pcre2pp::code::from_const(R"(^[^A-Z]+$)");
+    static const auto UPPER_RE
+        = lnav::pcre2pp::code::from_const(R"(^[^a-z]+$)");
+    static const auto CAMEL_RE
+        = lnav::pcre2pp::code::from_const(R"(^(?:[A-Z][a-z0-9]+)+$)");
+
+    if (LOWER_RE.find_in(*this).ignore_error().has_value()) {
+        return case_style::lower;
+    }
+    if (UPPER_RE.find_in(*this).ignore_error().has_value()) {
+        return case_style::upper;
+    }
+    if (CAMEL_RE.find_in(*this).ignore_error().has_value()) {
+        return case_style::camel;
+    }
+
+    return case_style::mixed;
+}
+
+std::string
+string_fragment::to_string_with_case_style(case_style style) const
+{
+    std::string retval;
+
+    switch (style) {
+        case case_style::lower: {
+            for (auto ch : *this) {
+                retval.append(1, std::tolower(ch));
+            }
+            break;
+        }
+        case case_style::upper: {
+            for (auto ch : *this) {
+                retval.append(1, std::toupper(ch));
+            }
+            break;
+        }
+        case case_style::camel: {
+            retval = this->to_string();
+            if (!this->empty()) {
+                retval[0] = toupper(retval[0]);
+            }
+            break;
+        }
+        case case_style::mixed: {
+            return this->to_string();
+        }
+    }
+
+    return retval;
+}
+
+std::string
+string_fragment::to_unquoted_string() const
+{
+    auto sub_sf = *this;
+
+    if (sub_sf.startswith("r") || sub_sf.startswith("u")) {
+        sub_sf = sub_sf.consume_n(1).value();
+    }
+    if (sub_sf.length() >= 2
+        && ((sub_sf.startswith("\"") && sub_sf.endswith("\""))
+            || (sub_sf.startswith("'") && sub_sf.endswith("'"))))
+    {
+        std::string retval;
+
+        sub_sf.sf_begin += 1;
+        sub_sf.sf_end -= 1;
+        retval.reserve(this->length());
+
+        auto in_escape = false;
+        for (auto ch : sub_sf) {
+            if (in_escape) {
+                switch (ch) {
+                    case 'n':
+                        retval.push_back('\n');
+                        break;
+                    case 't':
+                        retval.push_back('\t');
+                        break;
+                    case 'r':
+                        retval.push_back('\r');
+                        break;
+                    default:
+                        retval.push_back(ch);
+                        break;
+                }
+                in_escape = false;
+            } else if (ch == '\\') {
+                in_escape = true;
+            } else {
+                retval.push_back(ch);
+            }
+        }
+
+        return retval;
+    }
+
+    return this->to_string();
+}
+
+uint32_t
+string_fragment::front_codepoint() const
+{
+    size_t index = 0;
+    try {
+        return ww898::utf::utf8::read(
+            [this, &index]() { return this->data()[index++]; });
+    } catch (const std::runtime_error& e) {
+        return this->data()[0];
+    }
+}
+
+Result<ssize_t, const char*>
+string_fragment::codepoint_to_byte_index(ssize_t cp_index) const
+{
+    ssize_t retval = 0;
+
+    while (cp_index > 0) {
+        if (retval >= this->length()) {
+            return Err("index is beyond the end of the string");
+        }
+        auto ch_len = TRY(ww898::utf::utf8::char_size([this, retval]() {
+            return std::make_pair(this->data()[retval],
+                                  this->length() - retval - 1);
+        }));
+
+        retval += ch_len;
+        cp_index -= 1;
+    }
+
+    return Ok(retval);
+}
