@@ -34,7 +34,10 @@
 #include "config.h"
 #include "fmt/color.h"
 #include "itertools.hh"
+#include "lnav.console.into.hh"
 #include "log_level_enum.hh"
+#include "pcrepp/pcre2pp.hh"
+#include "snippet_highlighters.hh"
 #include "view_curses.hh"
 
 using namespace lnav::roles::literals;
@@ -42,6 +45,45 @@ using namespace lnav::roles::literals;
 namespace lnav {
 namespace console {
 
+snippet
+snippet::from_content_with_offset(intern_string_t src,
+                                  const attr_line_t& content,
+                                  size_t offset,
+                                  const std::string& errmsg)
+{
+    auto content_sf = string_fragment::from_str(content.get_string());
+    auto line_with_error = content_sf.find_boundaries_around(
+        offset, string_fragment::tag1{'\n'});
+    auto line_with_context = content_sf.find_boundaries_around(
+        offset, string_fragment::tag1{'\n'}, 3);
+    auto line_number = content_sf.sub_range(0, offset).count('\n');
+    auto erroff_in_line = offset - line_with_error.sf_begin;
+
+    attr_line_t pointer;
+
+    pointer.append(erroff_in_line, ' ')
+        .append("^ "_snippet_border)
+        .append(lnav::roles::error(errmsg))
+        .with_attr_for_all(VC_ROLE.value(role_t::VCR_QUOTED_CODE));
+
+    snippet retval;
+    retval.s_content
+        = content.subline(line_with_context.sf_begin,
+                          line_with_error.sf_end - line_with_context.sf_begin);
+    if (line_with_error.sf_end >= retval.s_content.get_string().size()) {
+        retval.s_content.append("\n");
+    }
+    retval.s_content.append(pointer).append(
+        content.subline(line_with_error.sf_end,
+                        line_with_context.sf_end - line_with_error.sf_end));
+    retval.s_location = source_location{
+        src,
+        static_cast<int32_t>(1 + line_number),
+    };
+
+    return retval;
+}
+
 user_message
 user_message::raw(const attr_line_t& al)
 {
@@ -235,13 +277,38 @@ curses_color_to_terminal_color(int curses_color)
     }
 }
 
+static bool
+get_no_color()
+{
+    return getenv("NO_COLOR") != nullptr;
+}
+
+static bool
+get_yes_color()
+{
+    return getenv("YES_COLOR") != nullptr;
+}
+
+static bool
+get_fd_tty(int fd)
+{
+    return isatty(fd);
+}
+
 void
 println(FILE* file, const attr_line_t& al)
 {
+    static const auto IS_NO_COLOR = get_no_color();
+    static const auto IS_YES_COLOR = get_yes_color();
+    static const auto IS_STDOUT_TTY = get_fd_tty(STDOUT_FILENO);
+    static const auto IS_STDERR_TTY = get_fd_tty(STDERR_FILENO);
+
     const auto& str = al.get_string();
 
-    if (getenv("NO_COLOR") != nullptr
-        || (!isatty(fileno(file)) && getenv("YES_COLOR") == nullptr))
+    if (IS_NO_COLOR || (file != stdout && file != stderr)
+        || (((file == stdout && !IS_STDOUT_TTY)
+             || (file == stderr && !IS_STDERR_TTY))
+            && !IS_YES_COLOR))
     {
         fmt::print(file, "{}\n", str);
         return;
@@ -270,6 +337,7 @@ println(FILE* file, const attr_line_t& al)
         auto line_style = fmt::text_style{};
         auto fg_style = fmt::text_style{};
         auto start = last_point.value();
+        nonstd::optional<std::string> href;
 
         for (const auto& attr : al.get_attrs()) {
             if (!attr.sa_range.contains(start)
@@ -279,7 +347,10 @@ println(FILE* file, const attr_line_t& al)
             }
 
             try {
-                if (attr.sa_type == &VC_BACKGROUND) {
+                if (attr.sa_type == &VC_HYPERLINK) {
+                    auto saw = string_attr_wrapper<std::string>(&attr);
+                    href = saw.get();
+                } else if (attr.sa_type == &VC_BACKGROUND) {
                     auto saw = string_attr_wrapper<int64_t>(&attr);
                     auto color_opt = curses_color_to_terminal_color(saw.get());
 
@@ -346,6 +417,9 @@ println(FILE* file, const attr_line_t& al)
                         case role_t::VCR_TEXT:
                         case role_t::VCR_IDENTIFIER:
                             break;
+                        case role_t::VCR_ALT_ROW:
+                            line_style |= fmt::emphasis::bold;
+                            break;
                         case role_t::VCR_SEARCH:
                             line_style |= fmt::emphasis::reverse;
                             break;
@@ -404,6 +478,7 @@ println(FILE* file, const attr_line_t& al)
                         case role_t::VCR_LIST_GLYPH:
                             line_style |= fmt::fg(fmt::terminal_color::yellow);
                             break;
+                        case role_t::VCR_INLINE_CODE:
                         case role_t::VCR_QUOTED_CODE:
                             default_fg_style
                                 = fmt::fg(fmt::terminal_color::white);
@@ -439,6 +514,9 @@ println(FILE* file, const attr_line_t& al)
             line_style |= default_bg_style;
         }
 
+        if (href) {
+            fmt::print(file, FMT_STRING("\x1b]8;;{}\x1b\\"), href.value());
+        }
         if (start < str.size()) {
             auto actual_end = std::min(str.size(), static_cast<size_t>(point));
             fmt::print(file,
@@ -446,6 +524,9 @@ println(FILE* file, const attr_line_t& al)
                        FMT_STRING("{}"),
                        str.substr(start, actual_end - start));
         }
+        if (href) {
+            fmt::print(file, FMT_STRING("\x1b]8;;\x1b\\"));
+        }
         last_point = point;
     }
     fmt::print(file, "\n");
@@ -462,5 +543,30 @@ print(FILE* file, const user_message& um)
     println(file, al);
 }
 
+user_message
+to_user_message(intern_string_t src, const lnav::pcre2pp::compile_error& ce)
+{
+    attr_line_t pcre_error_content{ce.ce_pattern};
+
+    lnav::snippets::regex_highlighter(pcre_error_content,
+                                      pcre_error_content.length(),
+                                      line_range{
+                                          0,
+                                          (int) pcre_error_content.length(),
+                                      });
+    pcre_error_content.append("\n")
+        .append(ce.ce_offset, ' ')
+        .append(lnav::roles::error("^ "))
+        .append(lnav::roles::error(ce.get_message()))
+        .with_attr_for_all(VC_ROLE.value(role_t::VCR_QUOTED_CODE));
+
+    return user_message::error(
+               attr_line_t()
+                   .append_quoted(ce.ce_pattern)
+                   .append(" is not a valid regular expression"))
+        .with_reason(ce.get_message())
+        .with_snippet(lnav::console::snippet::from(src, pcre_error_content));
+}
+
 }  // namespace console
 }  // namespace lnav
