@@ -32,16 +32,20 @@
 #include "base/enum_util.hh"
 #include "base/func_util.hh"
 #include "base/opt_util.hh"
+#include "bound_tags.hh"
 #include "config.h"
 #include "lnav.hh"
 #include "readline_highlighters.hh"
 #include "readline_possibilities.hh"
+#include "sql_util.hh"
 
 using namespace lnav::roles::literals;
 
 filter_sub_source::filter_sub_source(std::shared_ptr<readline_curses> editor)
     : fss_editor(editor)
 {
+    this->fss_editor->set_left(25);
+    this->fss_editor->set_width(-1);
     this->fss_editor->set_save_history(!(lnav_data.ld_flags & LNF_SECURE_MODE));
     this->fss_regex_context.set_highlighter(readline_regex_highlighter)
         .set_append_character(0);
@@ -71,7 +75,8 @@ filter_sub_source::list_input_handle_key(listview_curses& lv, int ch)
 {
     if (this->fss_editing) {
         switch (ch) {
-            case KEY_CTRL_RBRACKET:
+            case KEY_ESCAPE:
+            case KEY_CTRL(']'):
                 this->fss_editor->abort();
                 return true;
             default:
@@ -86,6 +91,7 @@ filter_sub_source::list_input_handle_key(listview_curses& lv, int ch)
             auto* tss = top_view->get_sub_source();
 
             tss->toggle_apply_filters();
+            top_view->reload_data();
             break;
         }
         case ' ': {
@@ -102,6 +108,7 @@ filter_sub_source::list_input_handle_key(listview_curses& lv, int ch)
             fs.set_filter_enabled(tf, !tf->is_enabled());
             tss->text_filters_changed();
             lv.reload_data();
+            top_view->reload_data();
             return true;
         }
         case 't': {
@@ -123,6 +130,7 @@ filter_sub_source::list_input_handle_key(listview_curses& lv, int ch)
 
             tss->text_filters_changed();
             lv.reload_data();
+            top_view->reload_data();
             return true;
         }
         case 'D': {
@@ -139,6 +147,7 @@ filter_sub_source::list_input_handle_key(listview_curses& lv, int ch)
             fs.delete_filter(tf->get_id());
             lv.reload_data();
             tss->text_filters_changed();
+            top_view->reload_data();
             return true;
         }
         case 'i': {
@@ -170,8 +179,7 @@ filter_sub_source::list_input_handle_key(listview_curses& lv, int ch)
             this->fss_editor->set_visible(true);
             this->fss_editor->set_y(
                 lv.get_y() + (int) (lv.get_selection() - lv.get_top()));
-            this->fss_editor->set_left(25);
-            this->fss_editor->set_width(this->tss_view->get_width() - 8 - 1);
+            this->fss_editor->window_change();
             this->fss_editor->focus(filter_lang_t::REGEX, "", "");
             this->fss_filter_state = true;
             ef->disable();
@@ -206,8 +214,7 @@ filter_sub_source::list_input_handle_key(listview_curses& lv, int ch)
             this->fss_editor->set_visible(true);
             this->fss_editor->set_y(
                 lv.get_y() + (int) (lv.get_selection() - lv.get_top()));
-            this->fss_editor->set_left(25);
-            this->fss_editor->set_width(this->tss_view->get_width() - 8 - 1);
+            this->fss_editor->window_change();
             this->fss_editor->focus(filter_lang_t::REGEX, "", "");
             this->fss_filter_state = true;
             ef->disable();
@@ -232,14 +239,14 @@ filter_sub_source::list_input_handle_key(listview_curses& lv, int ch)
                 : text_quoting::regex;
             add_view_text_possibilities(
                 this->fss_editor.get(), tf->get_lang(), "*", top_view, tq);
-            add_filter_expr_possibilities(
-                this->fss_editor.get(), filter_lang_t::SQL, "*");
+            if (top_view == &lnav_data.ld_views[LNV_LOG]) {
+                add_filter_expr_possibilities(
+                    this->fss_editor.get(), filter_lang_t::SQL, "*");
+            }
             this->fss_editor->set_window(lv.get_window());
             this->fss_editor->set_visible(true);
             this->fss_editor->set_y(
                 lv.get_y() + (int) (lv.get_selection() - lv.get_top()));
-            this->fss_editor->set_left(25);
-            this->fss_editor->set_width(this->tss_view->get_width() - 8 - 1);
             this->fss_editor->focus(tf->get_lang(), "");
             this->fss_editor->rewrite_line(0, tf->get_id().c_str());
             this->fss_filter_state = tf->is_enabled();
@@ -271,12 +278,15 @@ size_t
 filter_sub_source::text_line_count()
 {
     return (lnav_data.ld_view_stack.top() |
-            [](auto tc) {
-                text_sub_source* tss = tc->get_sub_source();
-                filter_stack& fs = tss->get_filters();
+                [](auto tc) -> nonstd::optional<size_t> {
+               text_sub_source* tss = tc->get_sub_source();
 
-                return nonstd::make_optional(fs.size());
-            })
+               if (tss == nullptr) {
+                   return nonstd::nullopt;
+               }
+               auto& fs = tss->get_filters();
+               return nonstd::make_optional(fs.size());
+           })
         .value_or(0);
 }
 
@@ -425,15 +435,15 @@ filter_sub_source::rl_change(readline_curses* rc)
             break;
         case filter_lang_t::REGEX: {
             auto regex_res
-                = pcrepp::shared_from_str(new_value, PCRE_CASELESS | PCRE_UTF8);
+                = lnav::pcre2pp::code::from(new_value, PCRE2_CASELESS);
 
             if (regex_res.isErr()) {
                 auto pe = regex_res.unwrapErr();
                 lnav_data.ld_filter_help_status_source.fss_error.set_value(
-                    "error: %s", pe.ce_msg);
+                    "error: %s", pe.get_message().c_str());
             } else {
                 auto& hm = top_view->get_highlights();
-                highlighter hl(regex_res.unwrap());
+                highlighter hl(regex_res.unwrap().to_shared());
                 auto role = tf->get_type() == text_filter::EXCLUDE
                     ? role_t::VCR_DIFF_DELETE
                     : role_t::VCR_DIFF_ADD;
@@ -508,21 +518,12 @@ filter_sub_source::rl_perform(readline_curses* rc)
         switch (tf->get_lang()) {
             case filter_lang_t::NONE:
             case filter_lang_t::REGEX: {
-                auto compile_res = pcrepp::shared_from_str(
-                    new_value, PCRE_CASELESS | PCRE_UTF8);
+                auto compile_res
+                    = lnav::pcre2pp::code::from(new_value, PCRE2_CASELESS);
 
                 if (compile_res.isErr()) {
                     auto ce = compile_res.unwrapErr();
-                    auto um = lnav::console::user_message::error(
-                                  "invalid regular expression")
-                                  .with_reason(ce.ce_msg)
-                                  .with_snippet(lnav::console::snippet::from(
-                                      INPUT_SRC, new_value));
-                    um.um_snippets.back()
-                        .s_content.append("\n")
-                        .append(ce.ce_offset, ' ')
-                        .append("^ "_comment)
-                        .append(lnav::roles::comment(ce.ce_msg));
+                    auto um = lnav::console::to_user_message(INPUT_SRC, ce);
                     lnav_data.ld_exec_context.ec_error_callback_stack.back()(
                         um);
                     this->rl_abort(rc);
@@ -530,11 +531,11 @@ filter_sub_source::rl_perform(readline_curses* rc)
                     tf->lf_deleted = true;
                     tss->text_filters_changed();
 
-                    auto pf
-                        = std::make_shared<pcre_filter>(tf->get_type(),
-                                                        new_value,
-                                                        tf->get_index(),
-                                                        compile_res.unwrap());
+                    auto pf = std::make_shared<pcre_filter>(
+                        tf->get_type(),
+                        new_value,
+                        tf->get_index(),
+                        compile_res.unwrap().to_shared());
 
                     *iter = pf;
                     tss->text_filters_changed();
@@ -585,6 +586,7 @@ filter_sub_source::rl_perform(readline_curses* rc)
     lnav_data.ld_filter_help_status_source.fss_prompt.clear();
     this->fss_editing = false;
     this->fss_editor->set_visible(false);
+    top_view->reload_data();
     this->tss_view->reload_data();
 }
 
@@ -650,7 +652,6 @@ filter_sub_source::rl_display_matches(readline_curses* rc)
     this->fss_match_view.set_x(rc->get_left() + rc->get_match_start());
     this->fss_match_view.set_width(width + 3);
     this->fss_match_view.set_needs_update();
-    this->fss_match_view.scroll_selection_into_view();
     this->fss_match_view.reload_data();
 }
 
