@@ -38,25 +38,18 @@
 #include "bound_tags.hh"
 #include "command_executor.hh"
 #include "config.h"
-#include "environ_vtab.hh"
 #include "field_overlay_source.hh"
 #include "lnav.hh"
 #include "lnav_config.hh"
-#include "lnav_util.hh"
-#include "log_data_helper.hh"
-#include "plain_text_source.hh"
-#include "readline_highlighters.hh"
 #include "shlex.hh"
 #include "sql_util.hh"
-#include "sysclip.hh"
-#include "termios_guard.hh"
 #include "xterm_mouse.hh"
 
 using namespace lnav::roles::literals;
 
 class logline_helper {
 public:
-    logline_helper(logfile_sub_source& lss) : lh_sub_source(lss) {}
+    explicit logline_helper(logfile_sub_source& lss) : lh_sub_source(lss) {}
 
     logline& move_to_msg_start()
     {
@@ -71,7 +64,7 @@ public:
         return (*lf)[cl];
     }
 
-    logline& current_line()
+    logline& current_line() const
     {
         content_line_t cl = this->lh_sub_source.at(this->lh_current_line);
         std::shared_ptr<logfile> lf = this->lh_sub_source.find(cl);
@@ -84,10 +77,11 @@ public:
         this->lh_string_attrs.clear();
         this->lh_line_values.clear();
         content_line_t cl = this->lh_sub_source.at(this->lh_current_line);
-        std::shared_ptr<logfile> lf = this->lh_sub_source.find(cl);
+        auto lf = this->lh_sub_source.find(cl);
         auto ll = lf->begin() + cl;
         auto format = lf->get_format();
         lf->read_full_message(ll, this->lh_line_values.lvv_sbr);
+        this->lh_line_values.lvv_sbr.erase_ansi();
         format->annotate(
             cl, this->lh_string_attrs, this->lh_line_values, false);
     }
@@ -165,6 +159,8 @@ handle_keyseq(const char* keyseq)
     ec.ec_error_callback_stack
         = lnav_data.ld_exec_context.ec_error_callback_stack;
     var_stack.push(std::map<std::string, scoped_value_t>());
+    // XXX push another so it doesn't look like interactive use
+    var_stack.push(std::map<std::string, scoped_value_t>());
     auto& vars = var_stack.top();
     vars["keyseq"] = keyseq;
     const auto& kc = iter->second;
@@ -205,10 +201,15 @@ handle_paging_key(int ch)
 
     textview_curses* tc = *lnav_data.ld_view_stack.top();
     exec_context& ec = lnav_data.ld_exec_context;
-    logfile_sub_source* lss = nullptr;
     text_sub_source* tc_tss = tc->get_sub_source();
     bookmarks<vis_line_t>::type& bm = tc->get_bookmarks();
 
+    if (tc->get_overlay_selection()) {
+        if (tc->handle_key(ch)) {
+            return true;
+        }
+    }
+
     auto keyseq = fmt::format(FMT_STRING("x{:02x}"), ch);
     if (handle_keyseq(keyseq.c_str())) {
         return true;
@@ -218,7 +219,8 @@ handle_paging_key(int ch)
         return true;
     }
 
-    lss = dynamic_cast<logfile_sub_source*>(tc->get_sub_source());
+    auto lss = dynamic_cast<logfile_sub_source*>(tc->get_sub_source());
+    auto text_accel_p = dynamic_cast<text_accel_source*>(tc->get_sub_source());
 
     /* process the command keystroke */
     switch (ch) {
@@ -250,7 +252,7 @@ handle_paging_key(int ch)
 
                 lnav_data.ld_last_view = nullptr;
                 if (src_view != nullptr && dst_view != nullptr) {
-                    src_view->time_for_row(top_tc->get_top()) |
+                    src_view->time_for_row(top_tc->get_selection()) |
                         [dst_view, tc](auto top_time) {
                             dst_view->row_for_time(top_time) |
                                 [tc](auto row) { tc->set_selection(row); };
@@ -302,7 +304,8 @@ handle_paging_key(int ch)
             } else {
                 alerter::singleton().chime("no more search hits to the right");
             }
-        } break;
+            break;
+        }
 
         case '<':
             if (tc->get_left() == 0) {
@@ -322,8 +325,8 @@ handle_paging_key(int ch)
 
         case 'f':
             if (tc == &lnav_data.ld_views[LNV_LOG]) {
-                bm[&logfile_sub_source::BM_FILES].next(tc->get_top()) |
-                    [&tc](auto vl) { tc->set_top(vl); };
+                bm[&logfile_sub_source::BM_FILES].next(tc->get_selection()) |
+                    [&tc](auto vl) { tc->set_selection(vl); };
             } else if (tc == &lnav_data.ld_views[LNV_TEXT]) {
                 textfile_sub_source& tss = lnav_data.ld_text_source;
 
@@ -336,8 +339,13 @@ handle_paging_key(int ch)
 
         case 'F':
             if (tc == &lnav_data.ld_views[LNV_LOG]) {
-                bm[&logfile_sub_source::BM_FILES].prev(tc->get_top()) |
-                    [&tc](auto vl) { tc->set_top(vl); };
+                bm[&logfile_sub_source::BM_FILES].prev(tc->get_selection()) |
+                    [&tc](auto vl) {
+                        // setting the selection for movement to previous file
+                        // marker instead of the top will move the cursor, too,
+                        // if needed.
+                        tc->set_selection(vl);
+                    };
             } else if (tc == &lnav_data.ld_views[LNV_TEXT]) {
                 textfile_sub_source& tss = lnav_data.ld_text_source;
 
@@ -371,31 +379,46 @@ handle_paging_key(int ch)
             break;
 
         case 'J':
-            if (lnav_data.ld_last_user_mark.find(tc)
-                    == lnav_data.ld_last_user_mark.end()
-                || !tc->is_line_visible(
-                    vis_line_t(lnav_data.ld_last_user_mark[tc])))
-            {
-                lnav_data.ld_select_start[tc] = tc->get_top();
-                lnav_data.ld_last_user_mark[tc] = tc->get_top();
+            if (tc->is_selectable()) {
+                if (tc->get_selection() >= 0_vl) {
+                    tc->toggle_user_mark(&textview_curses::BM_USER,
+                                         tc->get_selection());
+                    lnav_data.ld_select_start[tc] = tc->get_selection();
+                    lnav_data.ld_last_user_mark[tc] = tc->get_selection();
+                    if (tc->get_selection() + 1_vl < tc->get_inner_height()) {
+                        tc->set_selection(tc->get_selection() + 1_vl);
+                    }
+                }
             } else {
-                vis_line_t height;
-                unsigned long width;
-
-                tc->get_dimensions(height, width);
-                if (lnav_data.ld_last_user_mark[tc] > (tc->get_bottom() - 2)
-                    && tc->get_top() + height < tc->get_inner_height())
+                if (lnav_data.ld_last_user_mark.find(tc)
+                        == lnav_data.ld_last_user_mark.end()
+                    || !tc->is_line_visible(
+                        vis_line_t(lnav_data.ld_last_user_mark[tc])))
                 {
-                    tc->shift_top(1_vl);
+                    lnav_data.ld_select_start[tc] = tc->get_selection();
+                    lnav_data.ld_last_user_mark[tc] = tc->get_selection();
+                } else {
+                    vis_line_t height;
+                    unsigned long width;
+
+                    tc->get_dimensions(height, width);
+                    if (lnav_data.ld_last_user_mark[tc] > (tc->get_bottom() - 2)
+                        && tc->get_selection() + height
+                            < tc->get_inner_height())
+                    {
+                        tc->shift_top(1_vl);
+                    }
+                    if (lnav_data.ld_last_user_mark[tc] + 1
+                        >= tc->get_inner_height())
+                    {
+                        break;
+                    }
+                    lnav_data.ld_last_user_mark[tc] += 1;
                 }
-                if (lnav_data.ld_last_user_mark[tc] + 1
-                    >= tc->get_inner_height()) {
-                    break;
-                }
-                lnav_data.ld_last_user_mark[tc] += 1;
+                tc->toggle_user_mark(
+                    &textview_curses::BM_USER,
+                    vis_line_t(lnav_data.ld_last_user_mark[tc]));
             }
-            tc->toggle_user_mark(&textview_curses::BM_USER,
-                                 vis_line_t(lnav_data.ld_last_user_mark[tc]));
             tc->reload_data();
 
             lnav_data.ld_rl_view->set_alt_value(
@@ -410,14 +433,14 @@ handle_paging_key(int ch)
                 || !tc->is_line_visible(
                     vis_line_t(lnav_data.ld_last_user_mark[tc])))
             {
-                new_mark = tc->get_top();
+                new_mark = tc->get_selection();
             } else {
                 new_mark = lnav_data.ld_last_user_mark[tc];
             }
 
             tc->toggle_user_mark(&textview_curses::BM_USER,
                                  vis_line_t(new_mark));
-            if (new_mark == tc->get_top()) {
+            if (new_mark == tc->get_selection() && tc->get_top() > 0_vl) {
                 tc->shift_top(-1_vl);
             }
             if (new_mark > 0) {
@@ -426,21 +449,26 @@ handle_paging_key(int ch)
                 lnav_data.ld_last_user_mark[tc] = new_mark;
                 alerter::singleton().chime("no more lines to mark");
             }
-            lnav_data.ld_select_start[tc] = tc->get_top();
+            lnav_data.ld_select_start[tc] = tc->get_selection();
+            if (tc->is_selectable() && tc->get_selection() > 0_vl) {
+                tc->set_selection(tc->get_selection() - 1_vl);
+            }
             tc->reload_data();
 
             lnav_data.ld_rl_view->set_alt_value(
                 HELP_MSG_1(c, "to copy marked lines to the clipboard"));
-        } break;
+            break;
+        }
 
         case 'M':
             if (lnav_data.ld_last_user_mark.find(tc)
-                == lnav_data.ld_last_user_mark.end()) {
+                == lnav_data.ld_last_user_mark.end())
+            {
                 alerter::singleton().chime("no lines have been marked");
             } else {
-                int start_line = std::min((int) tc->get_top(),
+                int start_line = std::min((int) tc->get_selection(),
                                           lnav_data.ld_last_user_mark[tc] + 1);
-                int end_line = std::max((int) tc->get_top(),
+                int end_line = std::max((int) tc->get_selection(),
                                         lnav_data.ld_last_user_mark[tc] - 1);
 
                 tc->toggle_user_mark(&textview_curses::BM_USER,
@@ -468,20 +496,29 @@ handle_paging_key(int ch)
 #endif
 
         case 's':
-            if (lss) {
-                vis_line_t next_top = vis_line_t(tc->get_top() + 2);
+            if (text_accel_p && text_accel_p->is_time_offset_supported()) {
+                auto next_top = tc->get_selection() + 1_vl;
 
-                if (!lss->is_time_offset_enabled()) {
+                if (!tc->is_selectable()) {
+                    next_top += 1_vl;
+                }
+
+                if (!text_accel_p->is_time_offset_enabled()) {
                     lnav_data.ld_rl_view->set_alt_value(
                         HELP_MSG_1(T, "to disable elapsed-time mode"));
                 }
-                lss->set_time_offset(true);
+                text_accel_p->set_time_offset(true);
                 while (next_top < tc->get_inner_height()) {
-                    if (!lss->find_line(lss->at(next_top))->is_message()) {
-                    } else if (lss->get_line_accel_direction(next_top)
-                               == log_accel::A_DECEL) {
-                        --next_top;
-                        tc->set_top(next_top);
+                    if (!text_accel_p->text_accel_get_line(next_top)
+                             ->is_message())
+                    {
+                    } else if (text_accel_p->get_line_accel_direction(next_top)
+                               == log_accel::A_DECEL)
+                    {
+                        if (!tc->is_selectable()) {
+                            --next_top;
+                        }
+                        tc->set_selection(next_top);
                         break;
                     }
 
@@ -491,20 +528,28 @@ handle_paging_key(int ch)
             break;
 
         case 'S':
-            if (lss) {
-                vis_line_t next_top = tc->get_top();
+            if (text_accel_p && text_accel_p->is_time_offset_supported()) {
+                auto next_top = tc->get_selection();
 
-                if (!lss->is_time_offset_enabled()) {
+                if (tc->is_selectable() && next_top > 0_vl) {
+                    next_top -= 1_vl;
+                }
+                if (!text_accel_p->is_time_offset_enabled()) {
                     lnav_data.ld_rl_view->set_alt_value(
                         HELP_MSG_1(T, "to disable elapsed-time mode"));
                 }
-                lss->set_time_offset(true);
+                text_accel_p->set_time_offset(true);
                 while (0 <= next_top && next_top < tc->get_inner_height()) {
-                    if (!lss->find_line(lss->at(next_top))->is_message()) {
-                    } else if (lss->get_line_accel_direction(next_top)
-                               == log_accel::A_DECEL) {
-                        --next_top;
-                        tc->set_top(next_top);
+                    if (!text_accel_p->text_accel_get_line(next_top)
+                             ->is_message())
+                    {
+                    } else if (text_accel_p->get_line_accel_direction(next_top)
+                               == log_accel::A_DECEL)
+                    {
+                        if (!tc->is_selectable()) {
+                            --next_top;
+                        }
+                        tc->set_selection(next_top);
                         break;
                     }
 
@@ -532,33 +577,36 @@ handle_paging_key(int ch)
         case '0':
             if (lss) {
                 const int step = 24 * 60 * 60;
-                lss->time_for_row(tc->get_top()) | [lss, tc](auto first_time) {
-                    lss->find_from_time(roundup_size(first_time.tv_sec, step)) |
-                        [tc](auto line) { tc->set_top(line); };
-                };
+                lss->time_for_row(tc->get_selection()) |
+                    [lss, tc](auto first_time) {
+                        lss->find_from_time(
+                            roundup_size(first_time.tv_sec, step))
+                            | [tc](auto line) { tc->set_selection(line); };
+                    };
             }
             break;
 
         case ')':
             if (lss) {
-                lss->time_for_row(tc->get_top()) | [lss, tc](auto first_time) {
-                    time_t day = rounddown(first_time.tv_sec, 24 * 60 * 60);
-                    lss->find_from_time(day) | [tc](auto line) {
-                        if (line != 0_vl) {
-                            --line;
-                        }
-                        tc->set_top(line);
+                lss->time_for_row(tc->get_selection()) |
+                    [lss, tc](auto first_time) {
+                        time_t day = rounddown(first_time.tv_sec, 24 * 60 * 60);
+                        lss->find_from_time(day) | [tc](auto line) {
+                            if (line != 0_vl) {
+                                --line;
+                            }
+                            tc->set_selection(line);
+                        };
                     };
-                };
             }
             break;
 
         case 'D':
-            if (tc->get_top() == 0) {
+            if (tc->get_selection() == 0) {
                 alerter::singleton().chime(
                     "the top of the log has been reached");
             } else if (lss) {
-                lss->time_for_row(tc->get_top()) |
+                lss->time_for_row(tc->get_selection()) |
                     [lss, ch, tc](auto first_time) {
                         int step = ch == 'D' ? (24 * 60 * 60) : (60 * 60);
                         time_t top_time = first_time.tv_sec;
@@ -566,7 +614,7 @@ handle_paging_key(int ch)
                             if (line != 0_vl) {
                                 --line;
                             }
-                            tc->set_top(line);
+                            tc->set_selection(line);
                         };
                     };
 
@@ -576,11 +624,11 @@ handle_paging_key(int ch)
 
         case 'd':
             if (lss) {
-                lss->time_for_row(tc->get_top()) |
+                lss->time_for_row(tc->get_selection()) |
                     [ch, lss, tc](auto first_time) {
                         int step = ch == 'd' ? (24 * 60 * 60) : (60 * 60);
                         lss->find_from_time(first_time.tv_sec + step) |
-                            [tc](auto line) { tc->set_top(line); };
+                            [tc](auto line) { tc->set_selection(line); };
                     };
 
                 lnav_data.ld_rl_view->set_alt_value(HELP_MSG_1(/, "to search"));
@@ -589,16 +637,16 @@ handle_paging_key(int ch)
 
         case 'o':
         case 'O':
-            if (lss) {
+            if (lss != nullptr) {
                 logline_helper start_helper(*lss);
 
-                start_helper.lh_current_line = tc->get_top();
-                logline& start_line = start_helper.move_to_msg_start();
+                start_helper.lh_current_line = tc->get_selection();
+                auto& start_line = start_helper.move_to_msg_start();
                 start_helper.annotate();
 
-                struct line_range opid_range = find_string_attr_range(
-                    start_helper.lh_string_attrs, &logline::L_OPID);
-                if (!opid_range.is_valid()) {
+                const auto& opid_opt
+                    = start_helper.lh_line_values.lvv_opid_value;
+                if (!opid_opt) {
                     alerter::singleton().chime(
                         "Log message does not contain an opid");
                     lnav_data.ld_rl_view->set_attr_value(
@@ -615,7 +663,8 @@ handle_paging_key(int ch)
                     while (true) {
                         if (ch == 'o') {
                             if (++next_helper.lh_current_line
-                                >= tc->get_inner_height()) {
+                                >= tc->get_inner_height())
+                            {
                                 break;
                             }
                         } else {
@@ -631,19 +680,10 @@ handle_paging_key(int ch)
                             continue;
                         }
                         next_helper.annotate();
-                        struct line_range opid_next_range
-                            = find_string_attr_range(
-                                next_helper.lh_string_attrs, &logline::L_OPID);
-                        const char* start_opid
-                            = start_helper.lh_line_values.lvv_sbr.get_data_at(
-                                opid_range.lr_start);
-                        const char* next_opid
-                            = next_helper.lh_line_values.lvv_sbr.get_data_at(
-                                opid_next_range.lr_start);
-                        if (opid_range.length() != opid_next_range.length()
-                            || memcmp(
-                                   start_opid, next_opid, opid_range.length())
-                                != 0)
+                        const auto& next_opid_opt
+                            = next_helper.lh_line_values.lvv_opid_value;
+                        if (next_opid_opt
+                            && opid_opt.value() != next_opid_opt.value())
                         {
                             continue;
                         }
@@ -652,16 +692,14 @@ handle_paging_key(int ch)
                     }
                     if (found) {
                         lnav_data.ld_rl_view->set_value("");
-                        tc->set_top(next_helper.lh_current_line);
+                        tc->set_selection(next_helper.lh_current_line);
                     } else {
-                        const auto opid_str
-                            = start_helper.to_string(opid_range);
-
                         lnav_data.ld_rl_view->set_attr_value(
                             lnav::console::user_message::error(
                                 attr_line_t(
                                     "No more messages found with opid: ")
-                                    .append(lnav::roles::symbol(opid_str)))
+                                    .append(
+                                        lnav::roles::symbol(opid_opt.value())))
                                 .to_attr_line());
                         alerter::singleton().chime(
                             "no more messages found with opid");
@@ -670,22 +708,6 @@ handle_paging_key(int ch)
             }
             break;
 
-        case 'p':
-            if (tc == &lnav_data.ld_views[LNV_LOG]) {
-                auto* fos = dynamic_cast<field_overlay_source*>(
-                    tc->get_overlay_source());
-                fos->fos_contexts.top().c_show
-                    = !fos->fos_contexts.top().c_show;
-                tc->set_needs_update();
-            } else if (tc == &lnav_data.ld_views[LNV_DB]) {
-                auto* dos = dynamic_cast<db_overlay_source*>(
-                    tc->get_overlay_source());
-
-                dos->dos_active = !dos->dos_active;
-                tc->set_needs_update();
-            }
-            break;
-
         case 't':
             if (lnav_data.ld_text_source.current_file() == nullptr) {
                 alerter::singleton().chime("No text files loaded");
@@ -698,15 +720,6 @@ handle_paging_key(int ch)
             }
             break;
 
-        case 'T':
-            lnav_data.ld_log_source.toggle_time_offset();
-            if (lss && lss->is_time_offset_enabled()) {
-                lnav_data.ld_rl_view->set_alt_value(HELP_MSG_2(
-                    s, S, "to move forward/backward through slow downs"));
-            }
-            tc->reload_data();
-            break;
-
         case 'I': {
             auto& hist_tc = lnav_data.ld_views[LNV_HISTOGRAM];
 
@@ -715,12 +728,14 @@ handle_paging_key(int ch)
                     = dynamic_cast<text_time_translator*>(tc->get_sub_source());
 
                 if (src_view != nullptr) {
-                    src_view->time_for_row(tc->get_top()) | [](auto log_top) {
-                        lnav_data.ld_hist_source2.row_for_time(log_top) |
-                            [](auto row) {
-                                lnav_data.ld_views[LNV_HISTOGRAM].set_top(row);
-                            };
-                    };
+                    src_view->time_for_row(tc->get_selection()) |
+                        [](auto log_top) {
+                            lnav_data.ld_hist_source2.row_for_time(log_top) |
+                                [](auto row) {
+                                    lnav_data.ld_views[LNV_HISTOGRAM]
+                                        .set_selection(row);
+                                };
+                        };
                 }
             } else {
                 lnav_data.ld_view_stack.top() | [&](auto top_tc) {
@@ -730,23 +745,24 @@ handle_paging_key(int ch)
                     if (dst_view != nullptr) {
                         auto& hs = lnav_data.ld_hist_source2;
                         auto hist_top_time_opt
-                            = hs.time_for_row(hist_tc.get_top());
+                            = hs.time_for_row(hist_tc.get_selection());
                         auto curr_top_time_opt
-                            = dst_view->time_for_row(top_tc->get_top());
+                            = dst_view->time_for_row(top_tc->get_selection());
                         if (hist_top_time_opt && curr_top_time_opt
                             && hs.row_for_time(hist_top_time_opt.value())
                                 != hs.row_for_time(curr_top_time_opt.value()))
                         {
                             dst_view->row_for_time(hist_top_time_opt.value()) |
                                 [top_tc](auto new_top) {
-                                    top_tc->set_top(new_top);
+                                    top_tc->set_selection(new_top);
                                     top_tc->set_needs_update();
                                 };
                         }
                     }
                 };
             }
-        } break;
+            break;
+        }
 
         case 'V': {
             textview_curses* db_tc = &lnav_data.ld_views[LNV_DB];
@@ -761,7 +777,7 @@ handle_paging_key(int ch)
 
                 if (log_line_index) {
                     fmt::memory_buffer linestr;
-                    int line_number = (int) tc->get_top();
+                    int line_number = (int) tc->get_selection();
                     unsigned int row;
 
                     fmt::format_to(std::back_inserter(linestr),
@@ -771,17 +787,18 @@ handle_paging_key(int ch)
                     for (row = 0; row < dls.dls_rows.size(); row++) {
                         if (strcmp(dls.dls_rows[row][log_line_index.value()],
                                    linestr.data())
-                            == 0) {
+                            == 0)
+                        {
                             vis_line_t db_line(row);
 
-                            db_tc->set_top(db_line);
+                            db_tc->set_selection(db_line);
                             db_tc->set_needs_update();
                             break;
                         }
                     }
                 }
             } else if (db_tc->get_inner_height() > 0) {
-                int db_row = db_tc->get_top();
+                int db_row = db_tc->get_selection();
                 tc = &lnav_data.ld_views[LNV_LOG];
                 auto log_line_index = dls.column_name_to_index("log_line");
 
@@ -797,7 +814,7 @@ handle_paging_key(int ch)
                                &line_number)
                         && line_number < tc->listview_rows(*tc))
                     {
-                        tc->set_top(vis_line_t(line_number));
+                        tc->set_selection(vis_line_t(line_number));
                         tc->set_needs_update();
                     }
                 } else {
@@ -809,10 +826,11 @@ handle_paging_key(int ch)
                         size_t col_len = strlen(col_value);
 
                         if (dts.scan(col_value, col_len, nullptr, &tm, tv)
-                            != nullptr) {
+                            != nullptr)
+                        {
                             lnav_data.ld_log_source.find_from_time(tv) |
                                 [tc](auto vl) {
-                                    tc->set_top(vl);
+                                    tc->set_selection(vl);
                                     tc->set_needs_update();
                                 };
                             break;
@@ -879,27 +897,18 @@ handle_paging_key(int ch)
             }
             break;
 
-        case 'x':
-            if (tc->toggle_hide_fields()) {
-                lnav_data.ld_rl_view->set_value("Showing hidden fields");
-            } else {
-                lnav_data.ld_rl_view->set_value("Hiding hidden fields");
-            }
-            tc->set_needs_update();
-            break;
-
         case 'r':
         case 'R':
-            if (lss) {
-                auto& last_time = injector::get<const relative_time&,
-                                                last_relative_time_tag>();
+            if (lss != nullptr) {
+                const auto& last_time = injector::get<const relative_time&,
+                                                      last_relative_time_tag>();
 
                 if (last_time.empty()) {
                     lnav_data.ld_rl_view->set_value(
                         "Use the 'goto' command to set the relative time to "
                         "move by");
                 } else {
-                    vis_line_t vl = tc->get_top(), new_vl;
+                    vis_line_t vl = tc->get_selection(), new_vl;
                     relative_time rt = last_time;
                     content_line_t cl;
                     struct exttm tm;
@@ -933,24 +942,18 @@ handle_paging_key(int ch)
                             done = true;
                         }
                     } while (!done);
-                    tc->set_top(vl);
+                    tc->set_selection(vl);
                     lnav_data.ld_rl_view->set_value(" " + rt.to_string());
                 }
             }
             break;
 
-        case KEY_CTRL_W:
-            execute_command(ec,
-                            lnav_data.ld_views[LNV_LOG].get_word_wrap()
-                                ? "disable-word-wrap"
-                                : "enable-word-wrap");
-            break;
-
-        case KEY_CTRL_P:
+        case KEY_CTRL('p'):
             lnav_data.ld_preview_hidden = !lnav_data.ld_preview_hidden;
             break;
 
         default:
+            log_debug("key sequence %x", ch);
             return false;
     }
     return true;
