@@ -132,7 +132,7 @@ update_tailer_description(
 
                 iter->second.ofd_description = remote_uname;
             }
-            fc.fc_name_to_errors.erase(netloc);
+            fc.fc_name_to_errors->writeAccess()->erase(netloc);
         });
 }
 
@@ -269,7 +269,7 @@ tailer::looper::complete_path(const network::path& path)
 static std::vector<std::string>
 create_ssh_args_from_config(const std::string& dest)
 {
-    auto& cfg = injector::get<const tailer::config&>();
+    const auto& cfg = injector::get<const tailer::config&>();
     std::vector<std::string> retval;
 
     retval.emplace_back(cfg.c_ssh_cmd);
@@ -787,9 +787,7 @@ tailer::looper::host_tailer::loop_body()
                 }
 
                 constexpr int64_t BUFFER_SIZE = 4 * 1024 * 1024;
-                auto_mem<unsigned char> buffer;
-
-                buffer = (unsigned char*) malloc(BUFFER_SIZE);
+                auto buffer = auto_mem<unsigned char>::malloc(BUFFER_SIZE);
                 auto remaining = pob.pob_length;
                 auto remaining_offset = pob.pob_offset;
                 tailer::hash_frag thf;
@@ -1151,11 +1149,11 @@ tailer::looper::report_error(std::string path, std::string msg)
     isc::to<main_looper&, services::main_t>().send([=](auto& mlooper) {
         file_collection fc;
 
-        fc.fc_name_to_errors.emplace(path,
-                                     file_error_info{
-                                         {},
-                                         msg,
-                                     });
+        fc.fc_name_to_errors->writeAccess()->emplace(path,
+                                                     file_error_info{
+                                                         {},
+                                                         msg,
+                                                     });
         update_active_files(fc);
         lnav_data.ld_active_files.fc_progress->writeAccess()->sp_tailers.erase(
             path);
