@@ -39,6 +39,7 @@
 
 #include "base/ansi_scrubber.hh"
 #include "base/attr_line.hh"
+#include "base/from_trait.hh"
 #include "base/itertools.hh"
 #include "base/lnav_log.hh"
 #include "config.h"
@@ -46,6 +47,20 @@
 #include "shlex.hh"
 #include "view_curses.hh"
 
+#if defined HAVE_NCURSESW_CURSES_H
+#    include <ncursesw/term.h>
+#elif defined HAVE_NCURSESW_H
+#    include <term.h>
+#elif defined HAVE_NCURSES_CURSES_H
+#    include <ncurses/term.h>
+#elif defined HAVE_NCURSES_H
+#    include <term.h>
+#elif defined HAVE_CURSES_H
+#    include <term.h>
+#else
+#    error "SysV or X/Open-compatible Curses header file required"
+#endif
+
 using namespace std::chrono_literals;
 
 const struct itimerval ui_periodic_timer::INTERVAL = {
@@ -109,9 +124,9 @@ struct utf_to_display_adjustment {
     int uda_offset;
 
     utf_to_display_adjustment(int utf_origin, int offset)
-        : uda_origin(utf_origin), uda_offset(offset){
-
-                                  };
+        : uda_origin(utf_origin), uda_offset(offset)
+    {
+    }
 };
 
 void
@@ -171,6 +186,18 @@ view_curses::mvwattrline(WINDOW* window,
                 break;
             }
 
+            case '\x1b':
+                expanded_line.append("\u238b");
+                utf_adjustments.emplace_back(lpc, -1);
+                char_index += 1;
+                break;
+
+            case '\b':
+                expanded_line.append("\u232b");
+                utf_adjustments.emplace_back(lpc, -1);
+                char_index += 1;
+                break;
+
             case '\r':
             case '\n':
                 expanded_line.push_back(' ');
@@ -237,7 +264,7 @@ view_curses::mvwattrline(WINDOW* window,
     }
 
     std::stable_sort(sa.begin(), sa.end());
-    for (auto iter = sa.begin(); iter != sa.end(); ++iter) {
+    for (auto iter = sa.cbegin(); iter != sa.cend(); ++iter) {
         auto attr_range = iter->sa_range;
 
         require(attr_range.lr_start >= 0);
@@ -246,23 +273,26 @@ view_curses::mvwattrline(WINDOW* window,
         if (!(iter->sa_type == &VC_ROLE || iter->sa_type == &VC_ROLE_FG
               || iter->sa_type == &VC_STYLE || iter->sa_type == &VC_GRAPHIC
               || iter->sa_type == &SA_LEVEL || iter->sa_type == &VC_FOREGROUND
-              || iter->sa_type == &VC_BACKGROUND))
+              || iter->sa_type == &VC_BACKGROUND
+              || iter->sa_type == &VC_BLOCK_ELEM))
         {
             continue;
         }
 
-        for (const auto& adj : utf_adjustments) {
-            // If the UTF adjustment is in the viewport, we need to adjust this
-            // attribute.
-            if (adj.uda_origin < iter->sa_range.lr_start) {
-                attr_range.lr_start += adj.uda_offset;
-            }
-        }
-
-        if (attr_range.lr_end != -1) {
+        if (attr_range.lr_unit == line_range::unit::bytes) {
             for (const auto& adj : utf_adjustments) {
-                if (adj.uda_origin < iter->sa_range.lr_end) {
-                    attr_range.lr_end += adj.uda_offset;
+                // If the UTF adjustment is in the viewport, we need to adjust
+                // this attribute.
+                if (adj.uda_origin < iter->sa_range.lr_start) {
+                    attr_range.lr_start += adj.uda_offset;
+                }
+            }
+
+            if (attr_range.lr_end != -1) {
+                for (const auto& adj : utf_adjustments) {
+                    if (adj.uda_origin < iter->sa_range.lr_end) {
+                        attr_range.lr_end += adj.uda_offset;
+                    }
                 }
             }
         }
@@ -319,9 +349,15 @@ view_curses::mvwattrline(WINDOW* window,
         if (attr_range.lr_start < attr_range.lr_end) {
             int awidth = attr_range.length();
             nonstd::optional<char> graphic;
+            nonstd::optional<wchar_t> block_elem;
 
             if (iter->sa_type == &VC_GRAPHIC) {
                 graphic = iter->sa_value.get<int64_t>();
+                attrs = text_attrs{};
+            } else if (iter->sa_type == &VC_BLOCK_ELEM) {
+                auto be = iter->sa_value.get<block_elem_t>();
+                block_elem = be.value;
+                attrs = vc.attrs_for_role(be.role);
             } else if (iter->sa_type == &VC_STYLE) {
                 attrs = iter->sa_value.get<text_attrs>();
             } else if (iter->sa_type == &SA_LEVEL) {
@@ -336,7 +372,7 @@ view_curses::mvwattrline(WINDOW* window,
                 attrs.ta_fg_color = role_attrs.ta_fg_color;
             }
 
-            if (graphic || !attrs.empty()) {
+            if (graphic || block_elem || !attrs.empty()) {
                 int x_pos = x + attr_range.lr_start;
                 int ch_width = std::min(
                     awidth, (line_width_chars - attr_range.lr_start));
@@ -381,6 +417,9 @@ view_curses::mvwattrline(WINDOW* window,
                         row_ch[lpc].chars[0] = graphic.value();
                         row_ch[lpc].attr |= A_ALTCHARSET;
                     }
+                    if (block_elem) {
+                        row_ch[lpc].chars[0] = block_elem.value();
+                    }
                     if (row_ch[lpc].attr & A_REVERSE
                         && attrs.ta_attrs & A_REVERSE)
                     {
@@ -481,10 +520,12 @@ static std::string COLOR_NAMES[] = {
 
 class color_listener : public lnav_config_listener {
 public:
+    color_listener() : lnav_config_listener(__FILE__) {}
+
     void reload_config(error_reporter& reporter) override
     {
         if (!view_colors::initialized) {
-            return;
+            view_colors::vc_active_palette = ansi_colors();
         }
 
         auto& vc = view_colors::singleton();
@@ -539,9 +580,8 @@ view_colors::init(bool headless)
     initialized = true;
 
     {
-        auto reporter = [](const void*, const lnav::console::user_message&) {
-
-        };
+        auto reporter
+            = [](const void*, const lnav::console::user_message& um) {};
 
         _COLOR_LISTENER.reload_config(reporter);
     }
@@ -585,15 +625,18 @@ attr_for_colors(nonstd::optional<short> fg, nonstd::optional<short> bg)
 view_colors::role_attrs
 view_colors::to_attrs(const lnav_theme& lt,
                       const positioned_property<style_config>& pp_sc,
-                      const positioned_property<style_config>& pp_fallback_sc,
                       lnav_config_listener::error_reporter& reporter)
 {
     const auto& sc = pp_sc.pp_value;
-    const auto& fallback_sc = pp_fallback_sc.pp_value;
     std::string fg1, bg1, fg_color, bg_color;
     intern_string_t role_class;
 
-    if (!pp_sc.pp_path.empty()) {
+    if (pp_sc.pp_path.empty()) {
+#if 0
+        // too slow to do this now
+        reporter(&sc.sc_color, lnav::console::user_message::warning(""));
+#endif
+    } else {
         auto role_class_path
             = ghc::filesystem::path(pp_sc.pp_path.to_string()).parent_path();
         auto inner = role_class_path.filename().string();
@@ -604,12 +647,13 @@ view_colors::to_attrs(const lnav_theme& lt,
     }
 
     fg1 = sc.sc_color;
-    if (fg1.empty()) {
-        fg1 = fallback_sc.sc_color;
-    }
     bg1 = sc.sc_background_color;
-    shlex(fg1).eval(fg_color, lt.lt_vars);
-    shlex(bg1).eval(bg_color, lt.lt_vars);
+    std::map<std::string, scoped_value_t> vars;
+    for (const auto& vpair : lt.lt_vars) {
+        vars[vpair.first] = vpair.second;
+    }
+    shlex(fg1).eval(fg_color, scoped_resolver{&vars});
+    shlex(bg1).eval(bg_color, scoped_resolver{&vars});
 
     auto fg = styling::color_unit::from_str(fg_color).unwrapOrElse(
         [&](const auto& msg) {
@@ -655,7 +699,7 @@ view_colors::init_roles(const lnav_theme& lt,
 
     /* Setup the mappings from roles to actual colors. */
     this->vc_role_attrs[lnav::enums::to_underlying(role_t::VCR_TEXT)]
-        = this->to_attrs(lt, lt.lt_style_text, lt.lt_style_text, reporter);
+        = this->to_attrs(lt, lt.lt_style_text, reporter);
 
     for (int ansi_fg = 0; ansi_fg < 8; ansi_fg++) {
         for (int ansi_bg = 0; ansi_bg < 8; ansi_bg++) {
@@ -668,24 +712,26 @@ view_colors::init_roles(const lnav_theme& lt,
             auto fg_str = fg_iter == lt.lt_vars.end() ? "" : fg_iter->second;
             auto bg_str = bg_iter == lt.lt_vars.end() ? "" : bg_iter->second;
 
-            auto rgb_fg = rgb_color::from_str(fg_str).unwrapOrElse(
-                [&](const auto& msg) {
-                    reporter(&fg_str,
-                             lnav::console::user_message::error(
-                                 attr_line_t("invalid color -- ")
-                                     .append_quoted(fg_str))
-                                 .with_reason(msg));
-                    return rgb_color{};
-                });
-            auto rgb_bg = rgb_color::from_str(bg_str).unwrapOrElse(
-                [&](const auto& msg) {
-                    reporter(&bg_str,
-                             lnav::console::user_message::error(
-                                 attr_line_t("invalid background color -- ")
-                                     .append_quoted(bg_str))
-                                 .with_reason(msg));
-                    return rgb_color{};
-                });
+            auto rgb_fg = from<rgb_color>(string_fragment::from_str(fg_str))
+                              .unwrapOrElse([&](const auto& msg) {
+                                  reporter(&fg_str,
+                                           lnav::console::user_message::error(
+                                               attr_line_t("invalid color -- ")
+                                                   .append_quoted(fg_str))
+                                               .with_reason(msg));
+                                  return rgb_color{};
+                              });
+            auto rgb_bg
+                = from<rgb_color>(string_fragment::from_str(bg_str))
+                      .unwrapOrElse([&](const auto& msg) {
+                          reporter(
+                              &bg_str,
+                              lnav::console::user_message::error(
+                                  attr_line_t("invalid background color -- ")
+                                      .append_quoted(bg_str))
+                                  .with_reason(msg));
+                          return rgb_color{};
+                      });
 
             short fg = vc_active_palette->match_color(lab_color(rgb_fg));
             short bg = vc_active_palette->match_color(lab_color(rgb_bg));
@@ -721,44 +767,43 @@ view_colors::init_roles(const lnav_theme& lt,
         .ra_class_name
         = intern_string::lookup("-lnav_styles_search");
     this->vc_role_attrs[lnav::enums::to_underlying(role_t::VCR_IDENTIFIER)]
-        = this->to_attrs(
-            lt, lt.lt_style_identifier, lt.lt_style_text, reporter);
+        = this->to_attrs(lt, lt.lt_style_identifier, reporter);
     this->vc_role_attrs[lnav::enums::to_underlying(role_t::VCR_OK)]
-        = this->to_attrs(lt, lt.lt_style_ok, lt.lt_style_text, reporter);
+        = this->to_attrs(lt, lt.lt_style_ok, reporter);
     this->vc_role_attrs[lnav::enums::to_underlying(role_t::VCR_INFO)]
-        = this->to_attrs(lt, lt.lt_style_info, lt.lt_style_text, reporter);
+        = this->to_attrs(lt, lt.lt_style_info, reporter);
     this->vc_role_attrs[lnav::enums::to_underlying(role_t::VCR_ERROR)]
-        = this->to_attrs(lt, lt.lt_style_error, lt.lt_style_text, reporter);
+        = this->to_attrs(lt, lt.lt_style_error, reporter);
     this->vc_role_attrs[lnav::enums::to_underlying(role_t::VCR_WARNING)]
-        = this->to_attrs(lt, lt.lt_style_warning, lt.lt_style_text, reporter);
+        = this->to_attrs(lt, lt.lt_style_warning, reporter);
     this->vc_role_attrs[lnav::enums::to_underlying(role_t::VCR_ALT_ROW)]
-        = this->to_attrs(lt, lt.lt_style_alt_text, lt.lt_style_text, reporter);
+        = this->to_attrs(lt, lt.lt_style_alt_text, reporter);
     this->vc_role_attrs[lnav::enums::to_underlying(role_t::VCR_HIDDEN)]
-        = this->to_attrs(lt, lt.lt_style_hidden, lt.lt_style_text, reporter);
+        = this->to_attrs(lt, lt.lt_style_hidden, reporter);
+    this->vc_role_attrs[lnav::enums::to_underlying(role_t::VCR_CURSOR_LINE)]
+        = this->to_attrs(lt, lt.lt_style_cursor_line, reporter);
+    this->vc_role_attrs[lnav::enums::to_underlying(
+        role_t::VCR_DISABLED_CURSOR_LINE)]
+        = this->to_attrs(lt, lt.lt_style_disabled_cursor_line, reporter);
     this->vc_role_attrs[lnav::enums::to_underlying(role_t::VCR_ADJUSTED_TIME)]
-        = this->to_attrs(
-            lt, lt.lt_style_adjusted_time, lt.lt_style_text, reporter);
+        = this->to_attrs(lt, lt.lt_style_adjusted_time, reporter);
     this->vc_role_attrs[lnav::enums::to_underlying(role_t::VCR_SKEWED_TIME)]
-        = this->to_attrs(
-            lt, lt.lt_style_skewed_time, lt.lt_style_text, reporter);
+        = this->to_attrs(lt, lt.lt_style_skewed_time, reporter);
     this->vc_role_attrs[lnav::enums::to_underlying(role_t::VCR_OFFSET_TIME)]
-        = this->to_attrs(
-            lt, lt.lt_style_offset_time, lt.lt_style_text, reporter);
+        = this->to_attrs(lt, lt.lt_style_offset_time, reporter);
+    this->vc_role_attrs[lnav::enums::to_underlying(role_t::VCR_FILE_OFFSET)]
+        = this->to_attrs(lt, lt.lt_style_file_offset, reporter);
     this->vc_role_attrs[lnav::enums::to_underlying(role_t::VCR_INVALID_MSG)]
-        = this->to_attrs(
-            lt, lt.lt_style_invalid_msg, lt.lt_style_text, reporter);
+        = this->to_attrs(lt, lt.lt_style_invalid_msg, reporter);
 
     this->vc_role_attrs[lnav::enums::to_underlying(role_t::VCR_STATUS)]
-        = this->to_attrs(lt, lt.lt_style_status, lt.lt_style_status, reporter);
+        = this->to_attrs(lt, lt.lt_style_status, reporter);
     this->vc_role_attrs[lnav::enums::to_underlying(role_t::VCR_WARN_STATUS)]
-        = this->to_attrs(
-            lt, lt.lt_style_warn_status, lt.lt_style_status, reporter);
+        = this->to_attrs(lt, lt.lt_style_warn_status, reporter);
     this->vc_role_attrs[lnav::enums::to_underlying(role_t::VCR_ALERT_STATUS)]
-        = this->to_attrs(
-            lt, lt.lt_style_alert_status, lt.lt_style_status, reporter);
+        = this->to_attrs(lt, lt.lt_style_alert_status, reporter);
     this->vc_role_attrs[lnav::enums::to_underlying(role_t::VCR_ACTIVE_STATUS)]
-        = this->to_attrs(
-            lt, lt.lt_style_active_status, lt.lt_style_status, reporter);
+        = this->to_attrs(lt, lt.lt_style_active_status, reporter);
     this->vc_role_attrs[lnav::enums::to_underlying(role_t::VCR_ACTIVE_STATUS2)]
         = role_attrs{
             this->vc_role_attrs[lnav::enums::to_underlying(
@@ -775,72 +820,57 @@ view_colors::init_roles(const lnav_theme& lt,
         .ra_reverse.ta_attrs
         |= A_BOLD;
     this->vc_role_attrs[lnav::enums::to_underlying(role_t::VCR_STATUS_TITLE)]
-        = this->to_attrs(
-            lt, lt.lt_style_status_title, lt.lt_style_status, reporter);
+        = this->to_attrs(lt, lt.lt_style_status_title, reporter);
     this->vc_role_attrs[lnav::enums::to_underlying(role_t::VCR_STATUS_SUBTITLE)]
-        = this->to_attrs(
-            lt, lt.lt_style_status_subtitle, lt.lt_style_status, reporter);
+        = this->to_attrs(lt, lt.lt_style_status_subtitle, reporter);
     this->vc_role_attrs[lnav::enums::to_underlying(role_t::VCR_STATUS_INFO)]
-        = this->to_attrs(
-            lt, lt.lt_style_status_info, lt.lt_style_status, reporter);
+        = this->to_attrs(lt, lt.lt_style_status_info, reporter);
 
     this->vc_role_attrs[lnav::enums::to_underlying(role_t::VCR_STATUS_HOTKEY)]
-        = this->to_attrs(
-            lt, lt.lt_style_status_hotkey, lt.lt_style_status, reporter);
+        = this->to_attrs(lt, lt.lt_style_status_hotkey, reporter);
     this->vc_role_attrs[lnav::enums::to_underlying(
         role_t::VCR_STATUS_TITLE_HOTKEY)]
-        = this->to_attrs(
-            lt, lt.lt_style_status_title_hotkey, lt.lt_style_status, reporter);
+        = this->to_attrs(lt, lt.lt_style_status_title_hotkey, reporter);
     this->vc_role_attrs[lnav::enums::to_underlying(
         role_t::VCR_STATUS_DISABLED_TITLE)]
-        = this->to_attrs(lt,
-                         lt.lt_style_status_disabled_title,
-                         lt.lt_style_status,
-                         reporter);
+        = this->to_attrs(lt, lt.lt_style_status_disabled_title, reporter);
 
     this->vc_role_attrs[lnav::enums::to_underlying(role_t::VCR_H1)]
-        = this->to_attrs(lt, lt.lt_style_header[0], lt.lt_style_text, reporter);
+        = this->to_attrs(lt, lt.lt_style_header[0], reporter);
     this->vc_role_attrs[lnav::enums::to_underlying(role_t::VCR_H2)]
-        = this->to_attrs(lt, lt.lt_style_header[1], lt.lt_style_text, reporter);
+        = this->to_attrs(lt, lt.lt_style_header[1], reporter);
     this->vc_role_attrs[lnav::enums::to_underlying(role_t::VCR_H3)]
-        = this->to_attrs(lt, lt.lt_style_header[2], lt.lt_style_text, reporter);
+        = this->to_attrs(lt, lt.lt_style_header[2], reporter);
     this->vc_role_attrs[lnav::enums::to_underlying(role_t::VCR_H4)]
-        = this->to_attrs(lt, lt.lt_style_header[3], lt.lt_style_text, reporter);
+        = this->to_attrs(lt, lt.lt_style_header[3], reporter);
     this->vc_role_attrs[lnav::enums::to_underlying(role_t::VCR_H5)]
-        = this->to_attrs(lt, lt.lt_style_header[4], lt.lt_style_text, reporter);
+        = this->to_attrs(lt, lt.lt_style_header[4], reporter);
     this->vc_role_attrs[lnav::enums::to_underlying(role_t::VCR_H6)]
-        = this->to_attrs(lt, lt.lt_style_header[5], lt.lt_style_text, reporter);
+        = this->to_attrs(lt, lt.lt_style_header[5], reporter);
     this->vc_role_attrs[lnav::enums::to_underlying(role_t::VCR_HR)]
-        = this->to_attrs(lt, lt.lt_style_hr, lt.lt_style_text, reporter);
+        = this->to_attrs(lt, lt.lt_style_hr, reporter);
     this->vc_role_attrs[lnav::enums::to_underlying(role_t::VCR_HYPERLINK)]
-        = this->to_attrs(lt, lt.lt_style_hyperlink, lt.lt_style_text, reporter);
+        = this->to_attrs(lt, lt.lt_style_hyperlink, reporter);
     this->vc_role_attrs[lnav::enums::to_underlying(role_t::VCR_LIST_GLYPH)]
-        = this->to_attrs(
-            lt, lt.lt_style_list_glyph, lt.lt_style_text, reporter);
+        = this->to_attrs(lt, lt.lt_style_list_glyph, reporter);
     this->vc_role_attrs[lnav::enums::to_underlying(role_t::VCR_BREADCRUMB)]
-        = this->to_attrs(
-            lt, lt.lt_style_breadcrumb, lt.lt_style_text, reporter);
+        = this->to_attrs(lt, lt.lt_style_breadcrumb, reporter);
     this->vc_role_attrs[lnav::enums::to_underlying(role_t::VCR_TABLE_BORDER)]
-        = this->to_attrs(
-            lt, lt.lt_style_table_border, lt.lt_style_text, reporter);
+        = this->to_attrs(lt, lt.lt_style_table_border, reporter);
     this->vc_role_attrs[lnav::enums::to_underlying(role_t::VCR_TABLE_HEADER)]
-        = this->to_attrs(
-            lt, lt.lt_style_table_header, lt.lt_style_text, reporter);
+        = this->to_attrs(lt, lt.lt_style_table_header, reporter);
     this->vc_role_attrs[lnav::enums::to_underlying(role_t::VCR_QUOTE_BORDER)]
-        = this->to_attrs(
-            lt, lt.lt_style_quote_border, lt.lt_style_text, reporter);
+        = this->to_attrs(lt, lt.lt_style_quote_border, reporter);
     this->vc_role_attrs[lnav::enums::to_underlying(role_t::VCR_QUOTED_TEXT)]
-        = this->to_attrs(
-            lt, lt.lt_style_quoted_text, lt.lt_style_text, reporter);
+        = this->to_attrs(lt, lt.lt_style_quoted_text, reporter);
     this->vc_role_attrs[lnav::enums::to_underlying(role_t::VCR_FOOTNOTE_BORDER)]
-        = this->to_attrs(
-            lt, lt.lt_style_footnote_border, lt.lt_style_text, reporter);
+        = this->to_attrs(lt, lt.lt_style_footnote_border, reporter);
     this->vc_role_attrs[lnav::enums::to_underlying(role_t::VCR_FOOTNOTE_TEXT)]
-        = this->to_attrs(
-            lt, lt.lt_style_footnote_text, lt.lt_style_text, reporter);
+        = this->to_attrs(lt, lt.lt_style_footnote_text, reporter);
     this->vc_role_attrs[lnav::enums::to_underlying(role_t::VCR_SNIPPET_BORDER)]
-        = this->to_attrs(
-            lt, lt.lt_style_snippet_border, lt.lt_style_text, reporter);
+        = this->to_attrs(lt, lt.lt_style_snippet_border, reporter);
+    this->vc_role_attrs[lnav::enums::to_underlying(role_t::VCR_INDENT_GUIDE)]
+        = this->to_attrs(lt, lt.lt_style_indent_guide, reporter);
 
     {
         positioned_property<style_config> stitch_sc;
@@ -851,7 +881,7 @@ view_colors::init_roles(const lnav_theme& lt,
             = lt.lt_style_status_title.pp_value.sc_background_color;
         this->vc_role_attrs[lnav::enums::to_underlying(
             role_t::VCR_STATUS_STITCH_TITLE_TO_SUB)]
-            = this->to_attrs(lt, stitch_sc, lt.lt_style_status, reporter);
+            = this->to_attrs(lt, stitch_sc, reporter);
     }
     {
         positioned_property<style_config> stitch_sc;
@@ -862,7 +892,7 @@ view_colors::init_roles(const lnav_theme& lt,
             = lt.lt_style_status_subtitle.pp_value.sc_background_color;
         this->vc_role_attrs[lnav::enums::to_underlying(
             role_t::VCR_STATUS_STITCH_SUB_TO_TITLE)]
-            = this->to_attrs(lt, stitch_sc, lt.lt_style_status, reporter);
+            = this->to_attrs(lt, stitch_sc, reporter);
     }
 
     {
@@ -874,7 +904,7 @@ view_colors::init_roles(const lnav_theme& lt,
             = lt.lt_style_status_subtitle.pp_value.sc_background_color;
         this->vc_role_attrs[lnav::enums::to_underlying(
             role_t::VCR_STATUS_STITCH_SUB_TO_NORMAL)]
-            = this->to_attrs(lt, stitch_sc, lt.lt_style_status, reporter);
+            = this->to_attrs(lt, stitch_sc, reporter);
     }
     {
         positioned_property<style_config> stitch_sc;
@@ -885,7 +915,7 @@ view_colors::init_roles(const lnav_theme& lt,
             = lt.lt_style_status.pp_value.sc_background_color;
         this->vc_role_attrs[lnav::enums::to_underlying(
             role_t::VCR_STATUS_STITCH_NORMAL_TO_SUB)]
-            = this->to_attrs(lt, stitch_sc, lt.lt_style_status, reporter);
+            = this->to_attrs(lt, stitch_sc, reporter);
     }
 
     {
@@ -897,7 +927,7 @@ view_colors::init_roles(const lnav_theme& lt,
             = lt.lt_style_status_title.pp_value.sc_background_color;
         this->vc_role_attrs[lnav::enums::to_underlying(
             role_t::VCR_STATUS_STITCH_TITLE_TO_NORMAL)]
-            = this->to_attrs(lt, stitch_sc, lt.lt_style_status, reporter);
+            = this->to_attrs(lt, stitch_sc, reporter);
     }
     {
         positioned_property<style_config> stitch_sc;
@@ -908,33 +938,24 @@ view_colors::init_roles(const lnav_theme& lt,
             = lt.lt_style_status.pp_value.sc_background_color;
         this->vc_role_attrs[lnav::enums::to_underlying(
             role_t::VCR_STATUS_STITCH_NORMAL_TO_TITLE)]
-            = this->to_attrs(lt, stitch_sc, lt.lt_style_status, reporter);
+            = this->to_attrs(lt, stitch_sc, reporter);
     }
 
     this->vc_role_attrs[lnav::enums::to_underlying(role_t::VCR_INACTIVE_STATUS)]
-        = this->to_attrs(
-            lt, lt.lt_style_inactive_status, lt.lt_style_status, reporter);
+        = this->to_attrs(lt, lt.lt_style_inactive_status, reporter);
     this->vc_role_attrs[lnav::enums::to_underlying(
         role_t::VCR_INACTIVE_ALERT_STATUS)]
-        = this->to_attrs(lt,
-                         lt.lt_style_inactive_alert_status,
-                         lt.lt_style_alert_status,
-                         reporter);
+        = this->to_attrs(lt, lt.lt_style_inactive_alert_status, reporter);
 
     this->vc_role_attrs[lnav::enums::to_underlying(role_t::VCR_POPUP)]
-        = this->to_attrs(lt, lt.lt_style_popup, lt.lt_style_text, reporter);
+        = this->to_attrs(lt, lt.lt_style_popup, reporter);
     this->vc_role_attrs[lnav::enums::to_underlying(role_t::VCR_FOCUSED)]
-        = this->to_attrs(
-            lt, lt.lt_style_focused, lt.lt_style_focused, reporter);
+        = this->to_attrs(lt, lt.lt_style_focused, reporter);
     this->vc_role_attrs[lnav::enums::to_underlying(
         role_t::VCR_DISABLED_FOCUSED)]
-        = this->to_attrs(lt,
-                         lt.lt_style_disabled_focused,
-                         lt.lt_style_disabled_focused,
-                         reporter);
+        = this->to_attrs(lt, lt.lt_style_disabled_focused, reporter);
     this->vc_role_attrs[lnav::enums::to_underlying(role_t::VCR_SCROLLBAR)]
-        = this->to_attrs(
-            lt, lt.lt_style_scrollbar, lt.lt_style_status, reporter);
+        = this->to_attrs(lt, lt.lt_style_scrollbar, reporter);
     {
         positioned_property<style_config> bar_sc;
 
@@ -943,7 +964,7 @@ view_colors::init_roles(const lnav_theme& lt,
             = lt.lt_style_scrollbar.pp_value.sc_background_color;
         this->vc_role_attrs[lnav::enums::to_underlying(
             role_t::VCR_SCROLLBAR_ERROR)]
-            = this->to_attrs(lt, bar_sc, lt.lt_style_alert_status, reporter);
+            = this->to_attrs(lt, bar_sc, reporter);
     }
     {
         positioned_property<style_config> bar_sc;
@@ -953,57 +974,62 @@ view_colors::init_roles(const lnav_theme& lt,
             = lt.lt_style_scrollbar.pp_value.sc_background_color;
         this->vc_role_attrs[lnav::enums::to_underlying(
             role_t::VCR_SCROLLBAR_WARNING)]
-            = this->to_attrs(lt, bar_sc, lt.lt_style_warn_status, reporter);
+            = this->to_attrs(lt, bar_sc, reporter);
     }
 
+    this->vc_role_attrs[lnav::enums::to_underlying(role_t::VCR_INLINE_CODE)]
+        = this->to_attrs(lt, lt.lt_style_inline_code, reporter);
     this->vc_role_attrs[lnav::enums::to_underlying(role_t::VCR_QUOTED_CODE)]
-        = this->to_attrs(
-            lt, lt.lt_style_quoted_code, lt.lt_style_text, reporter);
+        = this->to_attrs(lt, lt.lt_style_quoted_code, reporter);
     this->vc_role_attrs[lnav::enums::to_underlying(role_t::VCR_CODE_BORDER)]
-        = this->to_attrs(
-            lt, lt.lt_style_code_border, lt.lt_style_text, reporter);
+        = this->to_attrs(lt, lt.lt_style_code_border, reporter);
     this->vc_role_attrs[lnav::enums::to_underlying(role_t::VCR_KEYWORD)]
-        = this->to_attrs(lt, lt.lt_style_keyword, lt.lt_style_text, reporter);
+        = this->to_attrs(lt, lt.lt_style_keyword, reporter);
     this->vc_role_attrs[lnav::enums::to_underlying(role_t::VCR_STRING)]
-        = this->to_attrs(lt, lt.lt_style_string, lt.lt_style_text, reporter);
+        = this->to_attrs(lt, lt.lt_style_string, reporter);
     this->vc_role_attrs[lnav::enums::to_underlying(role_t::VCR_COMMENT)]
-        = this->to_attrs(lt, lt.lt_style_comment, lt.lt_style_text, reporter);
+        = this->to_attrs(lt, lt.lt_style_comment, reporter);
     this->vc_role_attrs[lnav::enums::to_underlying(role_t::VCR_DOC_DIRECTIVE)]
-        = this->to_attrs(
-            lt, lt.lt_style_doc_directive, lt.lt_style_text, reporter);
+        = this->to_attrs(lt, lt.lt_style_doc_directive, reporter);
     this->vc_role_attrs[lnav::enums::to_underlying(role_t::VCR_VARIABLE)]
-        = this->to_attrs(lt, lt.lt_style_variable, lt.lt_style_text, reporter);
+        = this->to_attrs(lt, lt.lt_style_variable, reporter);
     this->vc_role_attrs[lnav::enums::to_underlying(role_t::VCR_SYMBOL)]
-        = this->to_attrs(lt, lt.lt_style_symbol, lt.lt_style_text, reporter);
+        = this->to_attrs(lt, lt.lt_style_symbol, reporter);
+    this->vc_role_attrs[lnav::enums::to_underlying(role_t::VCR_NULL)]
+        = this->to_attrs(lt, lt.lt_style_null, reporter);
+    this->vc_role_attrs[lnav::enums::to_underlying(role_t::VCR_ASCII_CTRL)]
+        = this->to_attrs(lt, lt.lt_style_ascii_ctrl, reporter);
+    this->vc_role_attrs[lnav::enums::to_underlying(role_t::VCR_NON_ASCII)]
+        = this->to_attrs(lt, lt.lt_style_non_ascii, reporter);
     this->vc_role_attrs[lnav::enums::to_underlying(role_t::VCR_NUMBER)]
-        = this->to_attrs(lt, lt.lt_style_number, lt.lt_style_text, reporter);
+        = this->to_attrs(lt, lt.lt_style_number, reporter);
+    this->vc_role_attrs[lnav::enums::to_underlying(role_t::VCR_FUNCTION)]
+        = this->to_attrs(lt, lt.lt_style_function, reporter);
+    this->vc_role_attrs[lnav::enums::to_underlying(role_t::VCR_TYPE)]
+        = this->to_attrs(lt, lt.lt_style_type, reporter);
+    this->vc_role_attrs[lnav::enums::to_underlying(role_t::VCR_SEP_REF_ACC)]
+        = this->to_attrs(lt, lt.lt_style_sep_ref_acc, reporter);
 
     this->vc_role_attrs[lnav::enums::to_underlying(role_t::VCR_RE_SPECIAL)]
-        = this->to_attrs(
-            lt, lt.lt_style_re_special, lt.lt_style_text, reporter);
+        = this->to_attrs(lt, lt.lt_style_re_special, reporter);
     this->vc_role_attrs[lnav::enums::to_underlying(role_t::VCR_RE_REPEAT)]
-        = this->to_attrs(lt, lt.lt_style_re_repeat, lt.lt_style_text, reporter);
+        = this->to_attrs(lt, lt.lt_style_re_repeat, reporter);
     this->vc_role_attrs[lnav::enums::to_underlying(role_t::VCR_FILE)]
-        = this->to_attrs(lt, lt.lt_style_file, lt.lt_style_text, reporter);
+        = this->to_attrs(lt, lt.lt_style_file, reporter);
 
     this->vc_role_attrs[lnav::enums::to_underlying(role_t::VCR_DIFF_DELETE)]
-        = this->to_attrs(
-            lt, lt.lt_style_diff_delete, lt.lt_style_text, reporter);
+        = this->to_attrs(lt, lt.lt_style_diff_delete, reporter);
     this->vc_role_attrs[lnav::enums::to_underlying(role_t::VCR_DIFF_ADD)]
-        = this->to_attrs(lt, lt.lt_style_diff_add, lt.lt_style_text, reporter);
+        = this->to_attrs(lt, lt.lt_style_diff_add, reporter);
     this->vc_role_attrs[lnav::enums::to_underlying(role_t::VCR_DIFF_SECTION)]
-        = this->to_attrs(
-            lt, lt.lt_style_diff_section, lt.lt_style_text, reporter);
+        = this->to_attrs(lt, lt.lt_style_diff_section, reporter);
 
     this->vc_role_attrs[lnav::enums::to_underlying(role_t::VCR_LOW_THRESHOLD)]
-        = this->to_attrs(
-            lt, lt.lt_style_low_threshold, lt.lt_style_text, reporter);
+        = this->to_attrs(lt, lt.lt_style_low_threshold, reporter);
     this->vc_role_attrs[lnav::enums::to_underlying(role_t::VCR_MED_THRESHOLD)]
-        = this->to_attrs(
-            lt, lt.lt_style_med_threshold, lt.lt_style_text, reporter);
+        = this->to_attrs(lt, lt.lt_style_med_threshold, reporter);
     this->vc_role_attrs[lnav::enums::to_underlying(role_t::VCR_HIGH_THRESHOLD)]
-        = this->to_attrs(
-            lt, lt.lt_style_high_threshold, lt.lt_style_text, reporter);
+        = this->to_attrs(lt, lt.lt_style_high_threshold, reporter);
 
     for (auto level = static_cast<log_level_t>(LEVEL_UNKNOWN + 1);
          level < LEVEL__MAX;
@@ -1015,8 +1041,8 @@ view_colors::init_roles(const lnav_theme& lt,
             this->vc_level_attrs[level]
                 = role_attrs{text_attrs{}, text_attrs{}};
         } else {
-            this->vc_level_attrs[level] = this->to_attrs(
-                lt, level_iter->second, lt.lt_style_text, reporter);
+            this->vc_level_attrs[level]
+                = this->to_attrs(lt, level_iter->second, reporter);
         }
     }
 
@@ -1162,95 +1188,6 @@ view_colors::attrs_for_ident(const char* str, size_t len) const
     return retval;
 }
 
-lab_color::lab_color(const rgb_color& rgb)
-{
-    double r = rgb.rc_r / 255.0, g = rgb.rc_g / 255.0, b = rgb.rc_b / 255.0, x,
-           y, z;
-
-    r = (r > 0.04045) ? pow((r + 0.055) / 1.055, 2.4) : r / 12.92;
-    g = (g > 0.04045) ? pow((g + 0.055) / 1.055, 2.4) : g / 12.92;
-    b = (b > 0.04045) ? pow((b + 0.055) / 1.055, 2.4) : b / 12.92;
-
-    x = (r * 0.4124 + g * 0.3576 + b * 0.1805) / 0.95047;
-    y = (r * 0.2126 + g * 0.7152 + b * 0.0722) / 1.00000;
-    z = (r * 0.0193 + g * 0.1192 + b * 0.9505) / 1.08883;
-
-    x = (x > 0.008856) ? pow(x, 1.0 / 3.0) : (7.787 * x) + 16.0 / 116.0;
-    y = (y > 0.008856) ? pow(y, 1.0 / 3.0) : (7.787 * y) + 16.0 / 116.0;
-    z = (z > 0.008856) ? pow(z, 1.0 / 3.0) : (7.787 * z) + 16.0 / 116.0;
-
-    this->lc_l = (116.0 * y) - 16;
-    this->lc_a = 500.0 * (x - y);
-    this->lc_b = 200.0 * (y - z);
-}
-
-double
-lab_color::deltaE(const lab_color& other) const
-{
-    double deltaL = this->lc_l - other.lc_l;
-    double deltaA = this->lc_a - other.lc_a;
-    double deltaB = this->lc_b - other.lc_b;
-    double c1 = sqrt(this->lc_a * this->lc_a + this->lc_b * this->lc_b);
-    double c2 = sqrt(other.lc_a * other.lc_a + other.lc_b * other.lc_b);
-    double deltaC = c1 - c2;
-    double deltaH = deltaA * deltaA + deltaB * deltaB - deltaC * deltaC;
-    deltaH = deltaH < 0.0 ? 0.0 : sqrt(deltaH);
-    double sc = 1.0 + 0.045 * c1;
-    double sh = 1.0 + 0.015 * c1;
-    double deltaLKlsl = deltaL / (1.0);
-    double deltaCkcsc = deltaC / (sc);
-    double deltaHkhsh = deltaH / (sh);
-    double i = deltaLKlsl * deltaLKlsl + deltaCkcsc * deltaCkcsc
-        + deltaHkhsh * deltaHkhsh;
-    return i < 0.0 ? 0.0 : sqrt(i);
-}
-
-bool
-lab_color::operator<(const lab_color& rhs) const
-{
-    if (lc_l < rhs.lc_l)
-        return true;
-    if (rhs.lc_l < lc_l)
-        return false;
-    if (lc_a < rhs.lc_a)
-        return true;
-    if (rhs.lc_a < lc_a)
-        return false;
-    return lc_b < rhs.lc_b;
-}
-
-bool
-lab_color::operator>(const lab_color& rhs) const
-{
-    return rhs < *this;
-}
-
-bool
-lab_color::operator<=(const lab_color& rhs) const
-{
-    return !(rhs < *this);
-}
-
-bool
-lab_color::operator>=(const lab_color& rhs) const
-{
-    return !(*this < rhs);
-}
-
-bool
-lab_color::operator==(const lab_color& rhs) const
-{
-    return lc_l == rhs.lc_l && lc_a == rhs.lc_a && lc_b == rhs.lc_b;
-}
-
-bool
-lab_color::operator!=(const lab_color& rhs) const
-{
-    return !(rhs == *this);
-}
-
-#include <term.h>
-
 Result<screen_curses, std::string>
 screen_curses::create()
 {
