@@ -37,6 +37,7 @@
 #include <stdio.h>
 #include <stdlib.h>
 
+#include "base/injector.bind.hh"
 #include "base/injector.hh"
 #include "config.h"
 #include "data_parser.hh"
@@ -51,11 +52,15 @@
 
 const char* TMP_NAME = "scanned.tmp";
 
+static auto bound_file_options_hier
+    = injector::bind<lnav::safe_file_options_hier>::to_singleton();
+
 int
 main(int argc, char* argv[])
 {
     int c, retval = EXIT_SUCCESS;
     bool prompt = false, is_log = false, pretty_print = false;
+    bool scanner_details = false;
 
     {
         static auto builtin_formats
@@ -75,7 +80,7 @@ main(int argc, char* argv[])
         load_formats(paths, errors);
     }
 
-    while ((c = getopt(argc, argv, "pPl")) != -1) {
+    while ((c = getopt(argc, argv, "pPls")) != -1) {
         switch (c) {
             case 'p':
                 prompt = true;
@@ -89,6 +94,10 @@ main(int argc, char* argv[])
                 is_log = true;
                 break;
 
+            case 's':
+                scanner_details = true;
+                break;
+
             default:
                 retval = EXIT_FAILURE;
                 break;
@@ -152,9 +161,9 @@ main(int argc, char* argv[])
 
                 auto& root_formats = log_format::get_root_formats();
                 std::vector<std::shared_ptr<log_format>>::iterator iter;
-                std::vector<logline> index;
 
                 if (is_log) {
+                    std::vector<logline> index;
                     logfile_open_options loo;
                     auto open_res = logfile::open(argv[lpc], loo);
                     auto lf = open_res.unwrap();
@@ -162,12 +171,15 @@ main(int argc, char* argv[])
                     scan_batch_context sbc{allocator};
                     for (iter = root_formats.begin();
                          iter != root_formats.end() && !found;
-                         ++iter) {
+                         ++iter)
+                    {
                         line_info li = {{13}};
 
                         (*iter)->clear();
-                        if ((*iter)->scan(*lf, index, li, sbr, sbc)
-                            == log_format::SCAN_MATCH) {
+                        if ((*iter)
+                                ->scan(*lf, index, li, sbr, sbc)
+                                .is<log_format::scan_match>())
+                        {
                             format = (*iter)->specialized();
                             found = true;
                         }
@@ -182,13 +194,60 @@ main(int argc, char* argv[])
                 string_attrs_t sa;
 
                 if (format.get() != nullptr) {
-                    format->annotate(0, sa, ll_values);
+                    format->annotate(0, sa, ll_values, false);
                     body = find_string_attr_range(sa, &SA_BODY);
                 }
 
                 data_parser::TRACE_FILE = fopen("scanned.dpt", "w");
+                setvbuf(data_parser::TRACE_FILE, nullptr, _IONBF, 0);
 
-                data_scanner ds(sub_line, body.lr_start, sub_line.length());
+                data_scanner ds(sub_line, body.lr_start);
+
+                if (scanner_details) {
+                    fprintf(out,
+                            "             %s\n",
+                            ds.get_input().to_string().c_str());
+                    while (true) {
+                        auto tok_res = ds.tokenize2();
+
+                        if (!tok_res) {
+                            break;
+                        }
+
+                        fprintf(out,
+                                "%4s %3d:%-3d ",
+                                data_scanner::token2name(tok_res->tr_token),
+                                tok_res->tr_capture.c_begin,
+                                tok_res->tr_capture.c_end);
+                        size_t cap_index = 0;
+                        for (; cap_index < tok_res->tr_capture.c_end;
+                             cap_index++)
+                        {
+                            if (cap_index == tok_res->tr_capture.c_begin) {
+                                fputc('^', out);
+                            } else if (cap_index
+                                       == (tok_res->tr_capture.c_end - 1))
+                            {
+                                fputc('^', out);
+                            } else if (cap_index > tok_res->tr_capture.c_begin)
+                            {
+                                fputc('-', out);
+                            } else {
+                                fputc(' ', out);
+                            }
+                        }
+                        for (; cap_index < (int) ds.get_input().length();
+                             cap_index++)
+                        {
+                            fputc(' ', out);
+                        }
+
+                        auto sub = tok_res->to_string();
+                        fprintf(out, "  %s\n", sub.c_str());
+                    }
+                }
+
+                ds.reset();
                 data_parser dp(&ds);
                 std::string msg_format;
 
@@ -200,8 +259,7 @@ main(int argc, char* argv[])
                 fprintf(out, "format      :%s\n", msg_format.c_str());
 
                 if (pretty_print) {
-                    data_scanner ds2(
-                        sub_line, body.lr_start, sub_line.length());
+                    data_scanner ds2(sub_line, body.lr_start);
                     pretty_printer pp(&ds2, sa);
                     attr_line_t pretty_out;
 
@@ -224,7 +282,8 @@ main(int argc, char* argv[])
 
                 fclose(out);
 
-                sprintf(cmd, "diff -u %s %s", argv[lpc], TMP_NAME);
+                snprintf(
+                    cmd, sizeof(cmd), "diff -u %s %s", argv[lpc], TMP_NAME);
                 rc = system(cmd);
                 if (rc != 0) {
                     if (prompt) {
