@@ -1,6 +1,38 @@
 #! /bin/bash
 
+export TZ=UTC
 export YES_COLOR=1
+export DUMP_CRASH=1
+
+run_cap_test ${lnav_test} -n \
+    -c ":goto 0" \
+    -c ":convert-time-to bad-zone" \
+    ${test_dir}/logfile_access_log.0
+
+run_cap_test ${lnav_test} -n \
+    -c ":goto 0" \
+    -c ":convert-time-to America/Los_Angeles" \
+    ${test_dir}/logfile_access_log.0
+
+run_cap_test ${lnav_test} -nN \
+    -c ";SELECT ':echo Hello' || char(10) || ':echo World' AS cmds" \
+    -c ':eval ${cmds}'
+
+run_cap_test ${lnav_test} -nN \
+    -c ":cd /bad-dir"
+
+run_cap_test ${lnav_test} -nN \
+    -c ":cd ${test_dir}/logfile_access_log.0"
+
+run_cap_test ${lnav_test} -nN \
+    -c ":cd ${test_dir}" \
+    -c ":open logfile_access_log.0"
+
+run_cap_test ${lnav_test} -nN \
+    -e "echo Hello, World!"
+
+run_cap_test ${lnav_test} -nN \
+    -e "echo Hello, World! > /dev/stderr"
 
 run_cap_test ${lnav_test} -n \
     -c ":switch-to-view help" \
@@ -30,6 +62,10 @@ run_cap_test env TZ=UTC ${lnav_test} -n \
     -c ":unix-time 1612072409" \
     "${test_dir}/logfile_access_log.*"
 
+#run_cap_test env TZ=UTC ${lnav_test} -n \
+#    -c ":unix-time 16120724091612072409" \
+#    "${test_dir}/logfile_access_log.*"
+
 run_cap_test env TZ=UTC ${lnav_test} -n \
     -c ":current-time" \
     "${test_dir}/logfile_access_log.*"
@@ -57,6 +93,14 @@ run_cap_test ${lnav_test} -n -d /tmp/lnav.err \
     -c ":write-view-to -" \
     "${test_dir}/logfile_access_log.0"
 
+run_cap_test ${lnav_test} -n -d /tmp/lnav.err \
+    -c ":write-view-to --anonymize -" \
+    "${test_dir}/logfile_access_log.0"
+
+run_cap_test ${lnav_test} -n -d /tmp/lnav.err \
+    -c ":write-view-to --anonymize -" \
+    "${test_dir}/logfile_pretty.0"
+
 run_cap_test ${lnav_test} -n -d /tmp/lnav.err \
     -c ":filter-expr timeslice(:log_time_msecs, 'bad') is not null" \
     "${test_dir}/logfile_multiline.0"
@@ -174,7 +218,12 @@ run_cap_test ${lnav_test} -n \
     ${test_dir}/logfile_access_log.0
 
 run_cap_test ${lnav_test} -n \
-    -c ':goto 2022-06-16Tabc'
+    -c ':goto 2022-06-16Tabc' \
+    ${test_dir}/logfile_access_log.0
+
+run_cap_test ${lnav_test} -n \
+    -c ':goto 17:00:01.' \
+    ${test_dir}/logfile_access_log.0
 
 run_cap_test ${lnav_test} -n \
     -c ":goto 1" \
@@ -361,6 +410,12 @@ run_cap_test ${lnav_test} -n \
     -c ":pipe-line-to echo \$cs_uri_stem \$sc_status" \
     ${test_dir}/logfile_access_log.0
 
+run_cap_test ${lnav_test} -n \
+    -I ${test_dir} \
+    -c ":goto 5" \
+    -c ":pipe-line-to echo \$log_raw_text \$log_level \$user" \
+    ${test_dir}/logfile_json.json
+
 run_cap_test ${lnav_test} -n \
     -c ":switch-to-view pretty" \
     ${test_dir}/textfile_json_one_line.0
