@@ -69,14 +69,14 @@ rebuild_indexes_repeatedly()
 {
 }
 
+void
+wait_for_pipers(nonstd::optional<timeval>)
+{
+}
+
 readline_context::command_map_t lnav_commands;
 
 namespace injector {
-template<>
-void
-force_linking(sqlite_db_tag anno)
-{
-}
 
 template<>
 void
